@@ -7,7 +7,8 @@ pub struct CharacterConfig<'a> {
     pub format: &'a str,
     pub success_symbol: &'a str,
     pub error_symbol: &'a str,
-    pub vicmd_symbol: &'a str,
+    #[serde(alias = "vicmd_symbol")]
+    pub vimcmd_symbol: &'a str,
     pub vimcmd_visual_symbol: &'a str,
     pub vimcmd_replace_symbol: &'a str,
     pub vimcmd_replace_one_symbol: &'a str,
@@ -20,7 +21,7 @@ impl<'a> Default for CharacterConfig<'a> {
             format: "$symbol ",
             success_symbol: "[❯](bold green)",
             error_symbol: "[❯](bold red)",
-            vicmd_symbol: "[❮](bold green)",
+            vimcmd_symbol: "[❮](bold green)",
             vimcmd_visual_symbol: "[❮](bold yellow)",
             vimcmd_replace_symbol: "[❮](bold purple)",
             vimcmd_replace_one_symbol: "[❮](bold purple)",
