@@ -9,6 +9,7 @@ pub struct PulumiConfig<'a> {
     pub symbol: &'a str,
     pub style: &'a str,
     pub disabled: bool,
+    pub search_upwards: bool,
 }
 
 impl<'a> Default for PulumiConfig<'a> {
@@ -19,6 +20,7 @@ impl<'a> Default for PulumiConfig<'a> {
             symbol: " ",
             style: "bold 5",
             disabled: false,
+            search_upwards: true,
         }
     }
 }
